﻿using Google.Rpc;
using Hotline.KnowledgeBase;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using Hotline.Users;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class KnowledgeController : BaseController
    {
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeTempRepository _knowledgeTempRepository;
        private readonly IKnowledgePVRepository _knowledgePVRepository;
        private readonly ISessionContext _sessionContext;
        private readonly IKnowledgeService _knowledgeService;
        private readonly IMapper _mapper;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeTempRepository"></param>
        /// <param name="knowledgePVRepository"></param>
        /// <param name="sessionContext"></param>
        /// <param name="knowledgeService"></param>
        /// <param name=""></param>
        /// <param name="mapper"></param>
        public KnowledgeController(IKnowledgeRepository knowledgeRepository, IKnowledgeTempRepository knowledgeTempRepository,
            IKnowledgePVRepository knowledgePVRepository, ISessionContext sessionContext, IKnowledgeService knowledgeService
            , IMapper mapper)
        {
            _knowledgeRepository = knowledgeRepository;
            _knowledgeTempRepository = knowledgeTempRepository;
            _knowledgePVRepository = knowledgePVRepository;
            _sessionContext = sessionContext;
            _knowledgeService = knowledgeService;
            _mapper = mapper;
        }
        #region 草稿箱
        /// <summary>
        /// 草稿箱-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledgetemp-add")]
        public async Task<string> KnowledgeTempAdd([FromBody] AddKnowledgeDto dto)
        {
            var kn = _mapper.Map<KnowledgeTemp>(dto);
            if (string.IsNullOrEmpty(dto.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();
            kn.PageView = 0;
            kn.CreationTime = DateTime.Now;
            return await _knowledgeTempRepository.AddAsync(kn);
        }

        /// <summary>
        /// 草稿箱-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledgetemp-update")]
        public async Task KnowledgeTempUpdate([FromBody] UpdateKnowledgeDto dto)
        {
            var kn = _mapper.Map<KnowledgeTemp>(dto);
            await _knowledgeTempRepository.UpdateAsync(kn);
        }

        /// <summary>
        /// 草稿箱-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledgetemp-info")]
        public async Task<KnowledgeTemp> KnowledgeTempInfo(string Id)
        {
            var temp = await _knowledgeTempRepository.GetAsync(Id);
            if (temp is null)
                throw UserFriendlyException.SameMessage("知识草稿查询失败！");
            return temp;
        }

        /// <summary>
        /// 草稿箱删除
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledgetemp-delete")]
        public async Task KnowledgeTempDelete(string Id)
        {
            if (string.IsNullOrEmpty(Id))
                throw UserFriendlyException.SameMessage("Id不能为空！");

            await _knowledgeTempRepository.RemoveAsync(p => p.Id == Id);
        }

        #endregion

        #region 知识管理
        /// <summary>
        /// 知识库-知识修改-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledge-updateinfo")]
        [AllowAnonymous]
        public async Task<Knowledge> KnowledgeUpdateInfo(string Id)
        {
            var know = await _knowledgeRepository.GetAsync(Id);
            if (know is null)
            {
                //再次查询草稿箱数据
                var tempKnow = await _knowledgeTempRepository.GetAsync(Id);
                if (tempKnow is null)
                    throw UserFriendlyException.SameMessage("知识查询失败！");
                know = new Knowledge();
                _mapper.Map(tempKnow, know);
            }
            return know;
        }

        /// <summary>
        /// 知识库-查询详情-增加浏览量
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledge-info")]
        public async Task<Knowledge> KnowledgeInfo(string Id)
        {
            var know = await _knowledgeRepository.GetAsync(Id);
            if (know is null)
                throw UserFriendlyException.SameMessage("知识查询失败！");
            //写入浏览记录
            KnowledgePv knowledgePv = new()
            {
                CreationTime = DateTime.Now,
                KnowledgeId = know.Id
            };
            await _knowledgePVRepository.AddAsync(knowledgePv);

            //修改浏览量
            know.PageView++;
            await _knowledgeRepository.UpdateAsync(know);
            return know;
        }

        /// <summary>
        /// 知识库-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-add")]
        public async Task<string> KnowledgeAdd([FromBody] AddKnowledgeDto dto)
        {
            var kn = _mapper.Map<Knowledge>(dto);
            //知识草稿ID如果不为空，则表示是草稿箱的数据
            if (!string.IsNullOrEmpty(dto.Code))
            {
                //移除草稿箱数据
                await _knowledgeTempRepository.RemoveAsync(p => p.Code == dto.Code);
            }
            //如果草稿箱不存在数据，也就是Code为空，从新生成Code
            if (string.IsNullOrEmpty(kn.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();
            kn.PageView = 0;
            kn.CreationTime = DateTime.Now;
            kn.Status = EKnowledgeStatus.Auditing;

            return await _knowledgeRepository.AddAsync(kn);
            //调用流程创建


            //如果流程创建失败需要删除此条数据
            //await _knowledgeRepository.RemoveAsync(knowledge);
        }

        /// <summary>
        /// 知识库-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-update")]
        public async Task KnowledgeUpdate([FromBody] UpdateKnowledgeDto dto)
        {
            //知识修改逻辑
            //如果可以修改，此条数据不动，重新写入一条新数据，知识编号不变，版本号+1

            var know = await _knowledgeRepository.GetAsync(dto.Id);
            if (know is null || know.IsDeleted)
                throw UserFriendlyException.SameMessage("无效知识");
            if (know.Status == EKnowledgeStatus.Auditing)
                throw UserFriendlyException.SameMessage("审批中数据不能修改");
            var knowledge = _mapper.Map<Knowledge>(dto);
            knowledge.Code = know.Code;
            knowledge.PageView = 0;
            knowledge.CreationTime = DateTime.Now;
            knowledge.Status = EKnowledgeStatus.Auditing;
            knowledge.Id = "";
            knowledge.Version = know.Version + 1;
            //新增一条数据

            await _knowledgeRepository.AddAsync(knowledge);

            //调用流程创建


            //如果流程创建失败需要删除此条数据
            //await _knowledgeRepository.RemoveAsync(knowledge);
        }

        /// <summary>
        /// 审批操作
        /// </summary>
        /// <param name="Ids"></param>
        /// <param name="ApprovalStatus">审核状态：0：通过；1：不通过</param>
        /// <returns></returns>
        [HttpPost("knowledge-approvalhandle")]
        public async Task ApprovalHandle(List<string> Ids, string ApprovalStatus)
        {
            if (Ids is null || Ids.Count == 0)
                throw UserFriendlyException.SameMessage("数据不能为空");
            //审批操作
            foreach (var item in Ids)
            {
                var know = await _knowledgeRepository.GetAsync(item);
                if (know != null)
                {
                    //调用流程审批接口
                    //流程调用成功以后在处理以下业务

                    //处理数据
                    if (ApprovalStatus == "0")
                    {
                        know.Status = EKnowledgeStatus.Succeed;
                        know.IsOnShelf = true;
                        know.OnShelfTime = DateTime.Now;
                    }
                    else
                    {
                        know.Status = EKnowledgeStatus.Failed;
                    }
                    await _knowledgeRepository.UpdateAsync(know);
                }
            }
        }

        #endregion

        #region 知识列表
        /// <summary>
        /// 我的草稿箱
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-mydraftslist")]
        public async Task<PagedDto<KnowledgeDataDto>> MyDraftsList([FromBody] MyDraftsListPagedDto pagedDto)
        {
            var (total, items) = await _knowledgeTempRepository
             .Queryable()
             .Includes(it => it.User)
             .Includes(it => it.KnowledgeType)
             .Where(p => p.CreatorId == _sessionContext.RequiredUserId)
             .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
             .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
             .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
             .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
             .OrderByDescending(p => p.CreationTime)
             .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 测试列表查询数据
        /// </summary>
        /// <returns></returns>
        [HttpPost("knowledge-testlist")]
        [AllowAnonymous]
        public async Task<PagedDto<KnowledgeDataDto>> testlist([FromBody] KnowledgePagedListDto pagedDto)
        {
            RefAsync<int> total = 0;
            var temp = await _knowledgeRepository
               .Queryable()
               .LeftJoin<User>((o, cus) => o.CreatorId == cus.Id)
               .LeftJoin<SystemOrganize>((o, cus, sys) => o.CreatorOrgId == sys.Id)
               .LeftJoin<KnowledgeType>((o, cus, sys, kn) => o.KnowledgeTypeId == kn.Id)

               .Where((o, cus, sys, kn) => o.IsDeleted == false)
               .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), (o, cus, sys, kn) => o.Title.Contains(pagedDto.Title!))
               .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), (o, cus, sys, kn) => o.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
               .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), (o, cus, sys, kn) => o.HotspotId == pagedDto.HotspotId)
               .WhereIF(pagedDto.StartTime != null, (o, cus, sys, kn) => o.OnShelfTime >= pagedDto.StartTime)
               .WhereIF(pagedDto.EndTime != null, (o, cus, sys, kn) => o.OnShelfTime <= pagedDto.EndTime)
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), (o, cus, sys, kn) => cus.Name.Contains(pagedDto.CreateUserName!))
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), (o, cus, sys, kn) => sys.OrgName.Contains(pagedDto.CreateBMName!))

               .Select((o, cus, sys, kn) => new
               {
                   Id = o.Id,
                   index = SqlFunc.RowNumber($"{o.Version} desc ", $"{o.Code}"),
                   Code = o.Code,
                   Title = o.Title,
                   Version = o.Version,
                   CreationName = cus.Name,
                   CreationBMName = sys.OrgName,
                   KnowledgeTypeId = o.KnowledgeTypeId,
                   KnowledgeTypeName = kn.SpliceName,
                   IsDeleted = o.IsDeleted,
                   HotspotId = o.HotspotId,
                   OnShelfTime = o.OnShelfTime,
                   CreationTime = o.CreationTime,
                   PageView = o.PageView,
                   Status = o.Status,
                   OffShelfTime = o.OffShelfTime,
                   LastModificationTime = o.LastModificationTime
               })
           .MergeTable()//将结果合并成一个表
           .Where(d => d.index == 1)

          .ToPageListAsync(pagedDto.PageIndex, pagedDto.PageSize, total);
            return new PagedDto<KnowledgeDataDto>(total.Value, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(temp));
        }

        /// <summary>
        /// 全部知识
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-alllist")]
        public async Task<PagedDto<KnowledgeDataDto>> AllList([FromBody] AllKnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));

            RefAsync<int> total = 0;
            var temp = await _knowledgeRepository
               .Queryable()
               .LeftJoin<User>((o, cus) => o.CreatorId == cus.Id)
               .LeftJoin<SystemOrganize>((o, cus, sys) => o.CreatorOrgId == sys.Id)
               .LeftJoin<KnowledgeType>((o, cus, sys, kn) => o.KnowledgeTypeId == kn.Id)

               .Where((o, cus, sys, kn) => o.IsDeleted == false)
               .Where((o, cus, sys, kn) => bmList.Contains(o.CreatorOrgId!))
               .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), (o, cus, sys, kn) => o.Title.Contains(pagedDto.Title!))
               .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), (o, cus, sys, kn) => o.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
               .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), (o, cus, sys, kn) => o.HotspotId == pagedDto.HotspotId)
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), (o, cus, sys, kn) => cus.Name.Contains(pagedDto.CreateUserName!))
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), (o, cus, sys, kn) => sys.OrgName.Contains(pagedDto.CreateBMName!))

              //创建时间
              .WhereIF(pagedDto.StartTime != null, (o, cus, sys, kn) => o.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, (o, cus, sys, kn) => o.CreationTime <= pagedDto.EndTime)
               //上架时间
               .WhereIF(pagedDto.StartOnShelfTime != null, (o, cus, sys, kn) => o.OnShelfTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndOnShelfTime != null, (o, cus, sys, kn) => o.OnShelfTime <= pagedDto.EndTime)
               //下架时间
               .WhereIF(pagedDto.StartOffShelfTime != null, (o, cus, sys, kn) => o.OffShelfTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndOffShelfTime != null, (o, cus, sys, kn) => o.OffShelfTime <= pagedDto.EndTime)

               .Select((o, cus, sys, kn) => new
               {
                   Id = o.Id,
                   index = SqlFunc.RowNumber($"{o.Version} desc ", $"{o.Code}"),
                   Code = o.Code,
                   Title = o.Title,
                   Version = o.Version,
                   CreationName = cus.Name,
                   CreationBMName = sys.OrgName,
                   KnowledgeTypeId = o.KnowledgeTypeId,
                   KnowledgeTypeName = kn.SpliceName,
                   IsDeleted = o.IsDeleted,
                   HotspotId = o.HotspotId,
                   OnShelfTime = o.OnShelfTime,
                   CreationTime = o.CreationTime,
                   PageView = o.PageView,
                   Status = o.Status,
                   OffShelfTime = o.OffShelfTime,
                   LastModificationTime = o.LastModificationTime
               })
           .MergeTable()//将结果合并成一个表
           .Where(d => d.index == 1)

          .ToPageListAsync(pagedDto.PageIndex, pagedDto.PageSize, total);
            return new PagedDto<KnowledgeDataDto>(total.Value, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(temp));
        }

        /// <summary>
        /// 已上架
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-ontheshelf")]
        public async Task<PagedDto<KnowledgeDataDto>> OnTheShelf([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));

            RefAsync<int> total = 0;
            var temp = await _knowledgeRepository
               .Queryable()
               .LeftJoin<User>((o, cus) => o.CreatorId == cus.Id)
               .LeftJoin<SystemOrganize>((o, cus, sys) => o.CreatorOrgId == sys.Id)
               .LeftJoin<KnowledgeType>((o, cus, sys, kn) => o.KnowledgeTypeId == kn.Id)

               .Where((o, cus, sys, kn) => o.IsDeleted == false && o.IsOnShelf == true && o.Status == EKnowledgeStatus.Succeed)
               .Where((o, cus, sys, kn) => bmList.Contains(o.CreatorOrgId!))
               .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), (o, cus, sys, kn) => o.Title.Contains(pagedDto.Title!))
               .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), (o, cus, sys, kn) => o.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
               .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), (o, cus, sys, kn) => o.HotspotId == pagedDto.HotspotId)
               .WhereIF(pagedDto.StartTime != null, (o, cus, sys, kn) => o.OnShelfTime >= pagedDto.StartTime)
               .WhereIF(pagedDto.EndTime != null, (o, cus, sys, kn) => o.OnShelfTime <= pagedDto.EndTime)
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), (o, cus, sys, kn) => cus.Name.Contains(pagedDto.CreateUserName!))
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), (o, cus, sys, kn) => sys.OrgName.Contains(pagedDto.CreateBMName!))

               .Select((o, cus, sys, kn) => new
               {
                   Id = o.Id,
                   index = SqlFunc.RowNumber($"{o.Version} desc ", $"{o.Code}"),
                   Code = o.Code,
                   Title = o.Title,
                   Version = o.Version,
                   CreationName = cus.Name,
                   CreationBMName = sys.OrgName,
                   KnowledgeTypeId = o.KnowledgeTypeId,
                   KnowledgeTypeName = kn.SpliceName,
                   IsDeleted = o.IsDeleted,
                   HotspotId = o.HotspotId,
                   OnShelfTime = o.OnShelfTime,
                   CreationTime = o.CreationTime,
                   PageView = o.PageView,
                   Status = o.Status,
                   OffShelfTime = o.OffShelfTime,
                   LastModificationTime = o.LastModificationTime
               })
           .MergeTable()//将结果合并成一个表
           .Where(d => d.index == 1)
            .OrderByDescending(p => p.CreationTime)
          .ToPageListAsync(pagedDto.PageIndex, pagedDto.PageSize, total);
            return new PagedDto<KnowledgeDataDto>(total.Value, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(temp));
        }

        /// <summary>
        /// 已下架
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-offshelf")]
        public async Task<PagedDto<KnowledgeDataDto>> OffShelf([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));

            RefAsync<int> total = 0;
            var temp = await _knowledgeRepository
               .Queryable()
               .LeftJoin<User>((o, cus) => o.CreatorId == cus.Id)
               .LeftJoin<SystemOrganize>((o, cus, sys) => o.CreatorOrgId == sys.Id)
               .LeftJoin<KnowledgeType>((o, cus, sys, kn) => o.KnowledgeTypeId == kn.Id)

               .Where((o, cus, sys, kn) => o.IsDeleted == false && o.IsOnShelf == false && o.Status == EKnowledgeStatus.Succeed)//
               .Where((o, cus, sys, kn) => bmList.Contains(o.CreatorOrgId!))
               .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), (o, cus, sys, kn) => o.Title.Contains(pagedDto.Title!))
               .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), (o, cus, sys, kn) => o.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
               .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), (o, cus, sys, kn) => o.HotspotId == pagedDto.HotspotId)
               .WhereIF(pagedDto.StartTime != null, (o, cus, sys, kn) => o.OffShelfTime >= pagedDto.StartTime)
               .WhereIF(pagedDto.EndTime != null, (o, cus, sys, kn) => o.OffShelfTime <= pagedDto.EndTime)
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), (o, cus, sys, kn) => cus.Name.Contains(pagedDto.CreateUserName!))
                .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), (o, cus, sys, kn) => sys.OrgName.Contains(pagedDto.CreateBMName!))

               .Select((o, cus, sys, kn) => new
               {
                   Id = o.Id,
                   index = SqlFunc.RowNumber($"{o.Version} desc ", $"{o.Code}"),
                   Code = o.Code,
                   Title = o.Title,
                   Version = o.Version,
                   CreationName = cus.Name,
                   CreationBMName = sys.OrgName,
                   KnowledgeTypeId = o.KnowledgeTypeId,
                   KnowledgeTypeName = kn.SpliceName,
                   IsDeleted = o.IsDeleted,
                   HotspotId = o.HotspotId,
                   OnShelfTime = o.OnShelfTime,
                   CreationTime = o.CreationTime,
                   PageView = o.PageView,
                   Status = o.Status,
                   OffShelfTime = o.OffShelfTime,
                   LastModificationTime = o.LastModificationTime
               })
           .MergeTable()//将结果合并成一个表
           .Where(d => d.index == 1)
            .OrderByDescending(p => p.CreationTime)
          .ToPageListAsync(pagedDto.PageIndex, pagedDto.PageSize, total);
            return new PagedDto<KnowledgeDataDto>(total.Value, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(temp));

        }

        /// <summary>
        /// 审批中
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-underapproval")]
        public async Task<PagedDto<KnowledgeDataDto>> UnderApproval([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.Status == EKnowledgeStatus.Auditing && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 审批完成
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approved")]
        public async Task<PagedDto<KnowledgeDataDto>> Approved([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.Status == EKnowledgeStatus.Succeed && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 审批不通过
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvalfailed")]
        public async Task<PagedDto<KnowledgeDataDto>> ApprovalFailed([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.UserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.Status == EKnowledgeStatus.Failed && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        #endregion

        #region 知识检索
        /// <summary>
        /// 知识检索
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-knowretrieval")]
        public async Task<PagedDto<KnowledgeRetrievalDataDto>> KnowRetrieval([FromBody] KnowledgeRetrievalPagedListDto pagedDto)
        {
            if (string.IsNullOrEmpty(pagedDto.Keyword))
                throw UserFriendlyException.SameMessage("请输入关键字");

            //是否需要验证本部门以及下级部门的数据，不要注释下面这3行
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeRetrievalDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeRetrievalDataDto>>(new List<Knowledge>()));

            string a = EKnowledgeRetrievalType.All.ToString();
            RefAsync<int> total = 0;
            var temp = await _knowledgeRepository
              .Queryable()
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Succeed)
              .Where(p => bmList.Contains(p.CreatorOrgId!)) //是否需要验证本部门以及下级部门的数据，不要注释此行
              .WhereIF(!string.IsNullOrEmpty(pagedDto.RetrievalType) || pagedDto.RetrievalType == EKnowledgeRetrievalType.All.ToString(), 
                        d => d.Title.Contains(pagedDto.Keyword) || d.Content.Contains(pagedDto.Keyword) || d.Additions.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Title.ToString(), d => d.Title.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Content.ToString(), d => d.Content.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Enclosure.ToString(), d => d.Additions.Contains(pagedDto.Keyword))

              .Select(o => new
              {
                  Id = o.Id,
                  index = SqlFunc.RowNumber($"{o.Version} desc ", $"{o.Code}"),
                  Code = o.Code,
                  Title = o.Title,
                  Summary = o.Summary,
                  Content = o.Content,
                  PageView = o.PageView
              })

              .OrderByDescending(p => p.PageView)
             .ToPageListAsync(pagedDto.PageIndex, pagedDto.PageSize, total);

            return new PagedDto<KnowledgeRetrievalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeRetrievalDataDto>>(temp));
        }
        #endregion

        #region 审核管理
        /// <summary>
        /// 审核管理-全部
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvedalllist")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ApprovedAllList([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false)//&& p.AssignUserIds.Contains(_sessionContext.UserId!)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }

        /// <summary>
        /// 待审核
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-tobereviewed")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ToBeReviewed([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            //审核类型字段加在哪里
            //我审核通过的和我审核不通过的怎么查询
            //
            //还需要增加审核类型查询条件
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Auditing && p.AssignUserIds.Contains(_sessionContext.UserId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }

        /// <summary>
        /// 审核通过
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvedlist")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ApprovedList([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            //自己审批了哪些数据，通过什么方式去查询这个数据是属于自己的

            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Succeed)//&& p.AssignUserIds.Contains(pagedDto.UserId!)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }

        /// <summary>
        /// 审核不通过
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvedfailedlist")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ApprovedFailedList([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Failed)//&& p.AssignUserIds.Contains(pagedDto.UserId!)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }
        #endregion

    }
}
