﻿using Hotline.CallCenter.BlackLists;
using Hotline.Identity.Roles;
using Hotline.KnowledgeBase;
using Hotline.Share.Dtos.CallCenter;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Dtos.Roles;
using Hotline.Share.Dtos.Users;
using Hotline.Users;
using Mapster;

namespace Hotline.Application.Mappers
{
    public class MapperConfigs : IRegister
    {
        public void Register(TypeAdapterConfig config)
        {
            config.NewConfig<AddBlacklistDto, Blacklist>()
                .Ignore(d => d.Expired)
                .AfterMapping((s, t) => t.InitExpired());

            config.NewConfig<AddUserDto, User>()
                .Map(d => d.Name, x => x.Name ?? x.UserName);

            config.NewConfig<User, UserDto>()
                .Ignore(d => d.UserName)
                .Ignore(d => d.OrgName)
                .Ignore(d => d.Roles)
                .AfterMapping((s, d) =>
                {
                    d.UserName = s.Account?.UserName ?? string.Empty;
                    d.OrgName = s.Organization?.OrgName ?? string.Empty;
                    d.Roles = s.Account != null && s.Account.Roles.Any()
                        ? string.Join(',', s.Account.Roles.Select(d => d.DisplayName))
                        : string.Empty;
                })
                .Map(d => d.State, x => x.IsDeleted ? "已删除" : "正常");

            config.NewConfig<Role, RoleDto>()
                .Map(d => d.AccountIds, x => x.Accounts.Select(d => d.Id))
                .Map(d => d.State, x => x.IsDeleted ? "已删除" : "正常");

            config.NewConfig<KnowledgeApply, KnowledgeApplyPageDto>()
              .Map(d => d.CreationName, x => x.User.Name)
                .Map(d => d.CreationOrgName, x => x.SystemOrganize.OrgName);

            config.NewConfig<Knowledge, KnowledgeDataDto>()
              .Map(d => d.CreationName, x => x.User.Name)
                .Map(d => d.KnowledgeTypeName, x => x.KnowledgeType.SpliceName);

            config.NewConfig<Knowledge, KnowledgeApprovalDataDto>()
            .Map(d => d.CreationName, x => x.User.Name)
              .Map(d => d.KnowledgeTypeName, x => x.KnowledgeType.SpliceName)
                 .Map(d => d.CreateBMName, x => x.SystemOrganize.OrgName);
        }
    }
}
