﻿using Hotline.Share.Requests;

namespace Hotline.Share.Dtos.Knowledge
{
    /// <summary>
    /// 知识库基础
    /// </summary>
    public record KnowledgeBasePagedDto : PagedRequest
    {
        /// <summary>
        /// 知识类型
        /// </summary>
        public string? KnowledgeTypeId { get; set; }

        /// <summary>
        /// 知识标题
        /// </summary>
        public string? Title { get; set; }

        /// <summary>
        /// 创建时间-开始
        /// </summary>
        public DateTime? StartTime { get; set; }

        /// <summary>
        /// 创建时间-结束
        /// </summary>
        public DateTime? EndTime { get; set; }
    }

    /// <summary>
    /// 草稿箱
    /// </summary>
    public record MyDraftsListPagedDto() : KnowledgeBasePagedDto;

    /// <summary>
    /// 知识库
    /// </summary>
    /// <param name="CreateUserName">创建人</param>
    /// <param name="CreateBMName">创建部门</param>
    /// <param name="CreateBMId">创建部门ID</param>
    /// <param name="HotspotId">热点类型</param>
    public record KnowledgePagedListDto(string? CreateUserName, string? CreateBMName, string? CreateBMId, string? HotspotId) : KnowledgeBasePagedDto;

    /// <summary>
    /// 全部知识查询条件
    /// </summary>
    /// <param name="CreateUserName">创建人</param>
    /// <param name="CreateBMName">创建部门</param>
    /// <param name="CreateBMId">创建部门ID</param>
    /// <param name="HotspotId">热点类型</param>
    /// <param name="StartOnShelfTime">上架时间-开始</param>
    /// <param name="EndOnShelfTime">上架时间-结束</param>
    /// <param name="StartOffShelfTime">下架时间-开始</param>
    /// <param name="EndOffShelfTime">下架时间-开始</param>
    public record AllKnowledgePagedListDto(string? CreateUserName, string? CreateBMName, string? CreateBMId, string? HotspotId,
        DateTime? StartOnShelfTime, DateTime? EndOnShelfTime, DateTime? StartOffShelfTime, DateTime? EndOffShelfTime) : KnowledgeBasePagedDto;

    /// <summary>
    /// 审核管理
    /// </summary>
    /// <param name="AuditType"></param>
    /// <param name="Title"></param>
    /// <param name="CreateUserName"></param>
    /// <param name="CreateBMName"></param>
    public record KnowledgeApprovalPagedListDto(string? AuditType, string? Title, string? CreateUserName, string? CreateBMName) : PagedRequest;

    /// <summary>
    /// 知识检索
    /// </summary>
    /// <param name="RetrievalType"></param>
    public record KnowledgeRetrievalPagedListDto(string? RetrievalType) : PagedKeywordRequest;
}
