﻿using Hotline.Settings;
using Hotline.Users;
using Microsoft.IdentityModel.Tokens;
using XF.Domain.Dependency;

namespace Hotline.KnowledgeBase
{
    public class KnowledgeService : IKnowledgeService, IScopeDependency
    {
        private readonly ISystemOrganizeRepository _systemOrganizeRepository;
        private readonly IUserRepository _userRepository;
        private readonly IKnowledgeRepository _knowledgeRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="_systemOrganizeRepository"></param>
        public KnowledgeService(ISystemOrganizeRepository systemOrganizeRepository, IUserRepository userRepository, IKnowledgeRepository knowledgeRepository)
        {
            _systemOrganizeRepository = systemOrganizeRepository;
            _userRepository = userRepository;
            _knowledgeRepository = knowledgeRepository;
        }

        /// <summary>
        /// 根据用户ID获取当前用户所属部门及下级部门Id
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        public async Task<List<string>> GetChildBMId(string? userId)
        {
            var orgUser = await _userRepository.GetAsync(p => p.Id == userId);
            List<string> list = new();
            if (orgUser != null && !string.IsNullOrEmpty(orgUser.OrgId))
            {
                var tempList = await _knowledgeRepository.GetChild(orgUser.OrgId);
                list = tempList.Select(it => it.Id).ToList();
            }
            return list;
        }

    }
}

