﻿using Hotline.Share.Dtos.Knowledge;
using XF.Domain.Dependency;

namespace Hotline.KnowledgeBase
{
    public class KnowledgeTypeService : IKnowledgeTypeService, IScopeDependency
    {
        /// <summary>
        /// 
        /// </summary>
        private readonly IKnowledgeTypeRepository _knowledgeTypeRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeTypeRepository"></param>
        public KnowledgeTypeService(IKnowledgeTypeRepository knowledgeTypeRepository)
        {
            _knowledgeTypeRepository = knowledgeTypeRepository;
        }
        /// <summary>
        /// 查询所有子级
        /// </summary>
        /// <param name="treeDatas"></param>
        /// <param name="ID"></param>
        /// <param name="checkId"></param>
        /// <returns></returns>
        public List<TreeListDto> GetChildren(List<KnowledgeType> treeDatas, string ID, string checkId)
        {
            List<TreeListDto> nodeList = new();
            var children = treeDatas.Where(q => q.ParentId == ID);
            foreach (var dr in children)
            {
                TreeListDto node = new()
                {
                    name = dr.Name,
                    ParentID = dr.ParentId,
                    value = dr.Id,
                    IsEnable = dr.IsEnable
                };
                if (!string.IsNullOrEmpty(checkId) && checkId == dr.Id)
                {
                    node.selected = true;
                }
                node.children = GetChildren(treeDatas, node.value, checkId);
                nodeList.Add(node);
            }
            return nodeList;
        }

        /// <summary>
        /// 获取全称
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<string> GetFullName(string Id)
        {
            //获取全部父级名称
            var list = await GetParentNode(Id);
            //倒叙
            list.Reverse();
            //拆分
            return string.Join("-", list.ToArray());
        }

        /// <summary>
        /// 修改子级分类全称
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task UpdateChildNode(string Id)
        {
            //查询子分类
            var list = await GetChildNode(Id);
            if (list is not null && list.Count > 0)
            {
                foreach (var item in list)
                {
                    //获取全称
                    string FullName = await GetFullName(item.ParentId);
                    item.SpliceName = string.IsNullOrEmpty(FullName) ? item.Name : FullName + "-" + item.Name;
                    //修改全称
                    await _knowledgeTypeRepository.UpdateAsync(item);
                }
            }
        }

        /// <summary>
        /// 查询父级节点
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        private async Task<List<string>> GetParentNode(string Id)
        {
            List<string> list = new();
            var type = await _knowledgeTypeRepository.GetAsync(p => p.Id == Id);
            if (type != null)
            {
                list.Add(type.Name);
                list.AddRange(await GetParentNode(type.ParentId));
            }
            return list;
        }

        /// <summary>
        /// 查询子级节点
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        private async Task<List<KnowledgeType>> GetChildNode(string Id)
        {
            List<KnowledgeType> list = new();
            var typelist = await _knowledgeTypeRepository.QueryAsync(p => p.ParentId == Id);
            if (typelist != null)
            {
                foreach (var item in typelist)
                {
                    list.Add(item);
                    list.AddRange(await GetChildNode(item.Id));
                }
            }
            return list;
        }

    }
}
