﻿using Hotline.Share.Enums.Order;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.Orders
{
    /// <summary>
    /// 工单
    /// </summary>
    public partial class Order : PositionWorkflowEntity
    {
        #region 来电信息

        /// <summary>
        /// 来源渠道
        /// </summary>
        public EChannel Channel { get; set; }

        /// <summary>
        /// 服务人姓名（冗余）
        /// </summary>
        public string ServerName { get; set; }

        /// <summary>
        /// 服务人工号（冗余）
        /// </summary>
        public string StaffNo { get; set; }

        /// <summary>
        /// 来电/信人姓名
        /// </summary>
        public string FromName { get; set; }

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public EGender FromGender { get; set; }

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public EFromStatus FromStatus { get; set; }

        /// <summary>
        /// 证件类型
        /// </summary>
        public ELicenceType? Licence { get; set; }

        /// <summary>
        /// 证件号码
        /// </summary>
        public string? LicenceNo { get; set; }

        /// <summary>
        /// 年龄段
        /// </summary>
        public EAgeRange? AgeRange { get; set; }

        /// <summary>
        /// 联系电话
        /// </summary>
        [SugarColumn(ColumnDescription = "联系电话", IsNullable = true)]
        public string? Contact { get; set; }

        /// <summary>
        /// 是否接受短信，勾选校验手机号
        /// </summary>
        public bool AcceptSms { get; set; }

        /// <summary>
        /// 是否需要联系
        /// </summary>
        public bool NeedContact { get; set; }

        /// <summary>
        /// 工作单位(当“来电/信人身份”为“企业”时必填，其他情况非必填)
        /// </summary>
        [SugarColumn(ColumnDescription = "工作单位", IsNullable = true)]
        public string? Company { get; set; }

        #endregion

        #region 诉求信息

        /// <summary>
        /// 工单编码（202201010001）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 工单类型
        /// </summary>
        public EOrderType OrderType { get; set; }//todo

        /// <summary>
        /// 受理类型
        /// </summary>
        public EAcceptType AcceptType { get; set; }

        /// <summary>
        /// 紧急程度
        /// </summary>
        public EEmergencyLevel EmergencyLevel { get; set; } = EEmergencyLevel.Normal;

        public string Title { get; set; }

        public string HotspotId { get; set; }
        public string Hotspot { get; set; }
        public string HotspotSpliceName { get; set; }

        /// <summary>
        /// 事发时间
        /// </summary>
        public DateTime? IncidentTime { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        [SugarColumn(ColumnDataType = "varchar(2000)", IsJson = true)]
        public List<string> Additions { get; set; } = new();

        #endregion
    }

    /// <summary>
    /// 渠道为电话时存在字段
    /// </summary>
    public partial class Order
    {
        /// <summary>
        /// 来电号码
        /// </summary>
        public string? FromPhone { get; set; }

        /// <summary>
        /// 转接号码（转接来源）
        /// </summary>
        public string? TransferPhone { get; set; }


    }
}
